import { Response } from './index';
import { capitalize, isArray } from './lib';
import middlewares from './middlewares';
import app from './index';
import fs from 'fs';

class Router {

  routesArray;
  rootPath;
  controllersPath;
  actionName;
  controllerName;

  setControllersPath(path) {
    this.controllersPath = path;
  }

  routes(rootPath, routes) {
    if (isArray(routes)) {
      this.rootPath = rootPath;
      this.routesArray = routes;
    } else {
      throw new Error('routes in "Router" must be array.');
    }
  }

  route(req, res) {
    const {url, method} = req;

    this.routesArray.map(route => {
      route.method = route.method.toUpperCase();

      const response = new Response(res);

      const run = () => {
        if (route.middleWares && !route.action) {
          return middlewares(req, res, route.middleWares);
        } else if (route.middleWares && route.action) {
          throw new Error('params: action and middleWares cannot work together');
        }

        if (((url + '/') === `${this.rootPath}${route.path}` || url === `${this.rootPath}${route.path}`) && method === route.method) {
          let [name, action] = route.action.split('@');
          this.actionName = action;
          this.controllerName = capitalize(name);
          let Controller;

          try {
            Controller = new (require(`${this.controllersPath}/${name}`).default);
          } catch(e) {
            try {
              Controller = new (require(`${this.controllersPath}/${name}`)[this.controllerName]);
            } catch(e) {
              throw new Error(`${this.controllerName} controller contains error or cannot find "${this.controllersPath}/${name}" in ${app.get('base')}`);
            }
          }

          this.action(req, res, Controller[action].bind(Controller), response);
        }
      };

      if (route.guard) return route.guard({ req: req, res: res, next: run, response: response });

      run();
    });
  }

  action(req, res, method, response) {
    method({ req: req, res: res, response: response });
  }

}

export default new Router();
