import Router from './Router';
import Response from './Response';
import Controller from './Controller';

import http from 'http';
import logger from './logger';

export {
  Router,
  Response,
  Controller as BaseController,
};

class App {

  port = 6060;
  prefix = '';
  base;
  portMessage = true;

  run() {
    let routes, controllers;
    let base = this.base = `${process.cwd()}/${this.prefix}`;

    controllers = `${base}/controllers`;

    try {
      routes = require(`${base}/routes`).default;
    } catch(e) {
      try {
        routes = require(`${base}/routes`);
      } catch(e) {
        throw new Error(`cannot find ${base}/routes, or routes contains error`);
      }
    }


    try {
      this.port = require(`${base}/config`).port;
    } catch(e) {}

    Router.setControllersPath(controllers);

    if (this.portMessage) {
      this.portMessage = false;

      console.info(`=== Server Started ===`);
      console.info('');
      console.info(`  port: ${this.port}`);
      console.info('');
      console.info(`======================`);
      console.info('');
    }

    return http.createServer((req, res) => {
      logger(req, res);

      Router.route(req, res);
    }).listen(this.port);
  }

  set(key, val) {
    this[key] = val;
  }

  get(key) {
    if (this[key]) return this[key];
    else return undefined;
  }

}

export default new App();
